{This TISAPI example demonstrates the use of the database methods}
{Important Notes on Writing IDAPI-Enabled ISAPI Applications:
Every time a browser invokes your ISAPI dll, the server creates a new thread
out of your HttpExtensionProc.

In the Delphi 2.0 Database Applications Developer's Guide, Chapter 3, it
states that multi-threaded database applications require their own TSession
component for each thread.  These TSessions are managed through a TSessionList
component.  You should read Chapter 3 to get a better understanding of how this
sample works in regard to sessions created on the fly.
}

{NOTE: THIS CODE HAS NOT BEEN FULLY TESTED!  It is currently under construction}

library tsamp4;

uses
  SysUtils, Windows,
  {The following are needed for tables}
  DBTables, DB, BDE,
  {here are our ISAPI-related files}
  httpext, TISAPI1;  {DON'T use TISAPILt if you're using tables}

const
  SQLUserName = 'testuser';   //Change to a valid SQL user name
  SQLPassword = 'testpass';   //Change to the users password

{==============================================================================}
{ The following code helps deal with dynamically created sessions              }
{First we need to come up with a unique name for our session.  I suppose the
code below still doesn't guarantee uniqueness of the session name, but it's
a good start, and should work in most real-world application.}

{It may also be possible to use the ECB's ConnID value for the session name,
 but I haven't had a chance to try that yet.  Or perhaps I could use
 IntToStr(GetCurrentThreadID) for the session name }
Function MakeSession : TSession;
var
  SessName : String;
begin
  Randomize;
  SessName := 'ISAPISess' + IntToStr(Random(10000));
  while Sessions.FindSession(SessName) <> nil do
    SessName := 'ISAPISess' + IntToStr(Random(10000));

  result := Sessions.OpenSession(SessName);
  result.Active := False;
  result.SessionName := SessName;
  result.active := True;
end;

{==============================================================================}
{ HttpExtensionProc contains the guts of your ISAPI program.  Here's where you put
  almost all of your code}

Function HttpExtensionProc (var pECB : TEXTENSION_CONTROL_BLOCK) : DWORD; export; stdcall;
Const
  CrLf = ^M^J;  //Just handy to have around
var
  MyISAPI : TISAPI;

  MySession : TSession;  // Here are the database components we'll need
  MyDB     : TDatabase;
  MyTable  : TTable;

  Cntr : Integer;

begin
  MyISAPI := TISAPI.Create; //Create our TECB and assign its published data elements

  {Create our Session, Database and Table components}
  MySession := MakeSession;
  MyDB := TDatabase.Create(nil);
  MyTable := TTable.Create(nil);

  MyISAPI.SetECB(@pECB);    //Set the ECB pointer - Note the @ operator
  MyISAPI.SendStdHeader;    //Necessary for anything else to work

  // Start the page
  MyISAPI.SendLn('<HTML><HEAD><TITLE>TISAPI Database Test</TITLE></HEAD>');
  MyISAPI.SendLn('<BODY>');
  MyISAPI.SendLn('<H1>TISAPI Database Test</H1>');
  MyISAPI.SendLn('Remember...This is only a test<BR>');

  try
    MyDB.SessionName := MySession.SessionName;    // Set up the Database
    MyDB.AliasName := 'WebStats';  // This should be the BDE Alias for your database
    MyDB.DatabaseName := 'MyAlias';// This is how other components will refer to the database
    MyDB.LoginPrompt := False;
    MyDB.Params.Clear;
    MyDB.Params.Add('USER NAME='+SQLUserName); //SQLUserName and SQLPassword are set above
    MyDB.Params.Add('PASSWORD='+SQLPassword);

    MyTable.SessionName := MySession.SessionName;
    MyTable.DatabaseName := MyDB.DatabaseName;
    MyTable.TableName := 'dbo.inetlog';   // IIS Logs to this SQL table

    try
      MyISAPI.SendLn('Opening Table...<BR>');
      MyTable.Open;
      MyISAPI.SendLn('Table Opened<BR>');
    except
      on E: EDatabaseError do
        MyISAPI.SendLn('Table Open Failed!<BR>');
    end;

    MyISAPI.SendLn('<P>Some Table Info:<BR>');

    Cntr := 0;
    while Cntr < 5 do
    begin
      try
        MyISAPI.SendLn('Client Host:'+ MyTable.FieldByName('ClientHost').AsString+'<BR>');
        MyISAPI.SendLn('Log Time:'+ MyTable.FieldByName('LogTime').AsString+'<BR>');
        MyISAPI.SendLn('Target:'+ MyTable.FieldByName('target').AsString+'<P>');
        MyTable.Next;
      except
        on E: EDatabaseError do
          MyISAPI.SendLn('Table Reading Failed!<BR>');
      end;
      Cntr := Cntr + 1;
    end;

  finally
    MyISAPI.SendLn('</BODY></HTML>');

    MyTable.Close;              // Close everybody down
    MyDB.Close;
    MySession.Close;

    MyTable.Free;               // Free our resources
    MyDB.Free;
    MySession.Free; // This appears to remove the Session from the SessionList
    MyISAPI.Free;
  end;

  result := HSE_STATUS_SUCCESS;  // That's all folks
end;

{==============================================================================}
{GetExtensionVersion is required by the MS IIS.  Don't mess with it.}
Function GetExtensionVersion (var pVer : THSE_VERSION_INFO) : BOOL; export;  stdcall;
begin
  pVer.dwExtensionVersion := MAKELONG(HSE_VERSION_MINOR, HSE_VERSION_MAJOR);
  StrPCopy(pVer.lpszExtensionDesc, 'Delphi ISAPI DLL');
  result := True;
end;

exports
  HttpExtensionProc,
  GetExtensionVersion;

begin
end.

